@@ -2,8 +2,8 @@
  * MediaLibraryService.swift
  * VLC for iOS
  *****************************************************************************
- * Copyright © 2018 VideoLAN. All rights reserved.
- * Copyright © 2018 Videolabs
+ * Copyright © 2018-2022 VideoLAN. All rights reserved.
+ * Copyright © 2018-2022 Videolabs
  *
  * Authors: Soomin Lee <bubu # mikan.io>
  *
@@ -11,6 +11,8 @@
  *****************************************************************************/
 
 import CoreSpotlight
+import VLCMediaLibraryKit
+import UIKit
 
 // MARK: - Notification names
 
@@ -25,7 +27,7 @@ extension NSNotification {
 
 // MARK: -
 
-@objc protocol MediaLibraryObserver: class {
+@objc protocol MediaLibraryObserver: AnyObject {
     // Video
     @objc optional func medialibrary(_ medialibrary: MediaLibraryService,
                                      didAddVideos videos: [VLCMLMedia])
@@ -110,19 +112,9 @@ extension NSNotification {
     @objc optional func medialibraryDidStartRescan()
 }
 
-// MARK: -
-
-protocol MediaLibraryMigrationDelegate: class {
-    func medialibraryDidStartMigration(_ medialibrary: MediaLibraryService)
-
-    func medialibraryDidFinishMigration(_ medialibrary: MediaLibraryService)
-
-    func medialibraryDidStopMigration(_ medialibrary: MediaLibraryService)
-}
-
 // MARK: - Delegate for "Backup Media Library" setting
 
-@objc protocol MediaLibraryDeviceBackupDelegate: class {
+@objc protocol MediaLibraryDeviceBackupDelegate: AnyObject {
     @objc func medialibraryDidStartExclusion()
 
     @objc func medialibraryDidCompleteExclusion()
@@ -130,7 +122,7 @@ protocol MediaLibraryMigrationDelegate: class {
 
 // MARK: - Delegate for hiding Media Library
 
-@objc protocol MediaLibraryHidingDelegate: class {
+@objc protocol MediaLibraryHidingDelegate: AnyObject {
     @objc func medialibraryDidStartHiding()
 
     @objc func medialibraryDidCompleteHiding()
@@ -140,17 +132,17 @@ protocol MediaLibraryMigrationDelegate: class {
 
 class MediaLibraryService: NSObject {
     private static let databaseName: String = "medialibrary.db"
-    private static let migrationKey: String = "MigratedToVLCMediaLibraryKit"
     private static let didForceRescan: String = "MediaLibraryDidForceRescan"
 
-    private var didMigrate = UserDefaults.standard.bool(forKey: MediaLibraryService.migrationKey)
     private var didFinishDiscovery = false
 
+    private var desiredThumbnailWidth = UInt(320)
+    private var desiredThumbnailHeight = UInt(200)
+
     private(set) var observable = Observable<MediaLibraryObserver>()
 
     private(set) lazy var medialib = VLCMediaLibrary()
 
-    weak var migrationDelegate: MediaLibraryMigrationDelegate?
     @objc weak var deviceBackupDelegate: MediaLibraryDeviceBackupDelegate?
     @objc weak var hidingDelegate: MediaLibraryHidingDelegate?
 
@@ -159,13 +151,19 @@ class MediaLibraryService: NSObject {
 
     override init() {
         super.init()
-        medialib.delegate = self
         setupMediaLibrary()
+        medialib.delegate = self
         NotificationCenter.default.addObserver(self, selector: #selector(reload),
                                                name: .VLCNewFileAddedNotification, object: nil)
 
         NotificationCenter.default.addObserver(self, selector: #selector(handleWillEnterForegroundNotification),
                                                name: UIApplication.willEnterForegroundNotification, object: nil)
+
+        let screen = UIScreen.main
+        let cellSizeWidth = MovieCollectionViewCell.cellSizeForWidth(screen.bounds.width).width
+        let scaledCellWidth = cellSizeWidth * screen.scale
+        desiredThumbnailWidth = UInt(scaledCellWidth)
+        desiredThumbnailHeight = UInt(scaledCellWidth / 1.6)
     }
 }
 
@@ -237,120 +235,6 @@ private extension MediaLibraryService {
     }
 }
 
-// MARK: - Migration
-
-private extension MediaLibraryService {
-    func startMigrationIfNeeded() {
-        guard !didMigrate else {
-            return
-        }
-        migrationDelegate?.medialibraryDidStartMigration(self)
-
-        migrateToNewMediaLibrary() {
-            [unowned self] success in
-            if success {
-                self.migrationDelegate?.medialibraryDidFinishMigration(self)
-            } else {
-                self.migrationDelegate?.medialibraryDidStopMigration(self)
-            }
-        }
-    }
-
-    func migrateMedia(_ oldMedialibrary: MLMediaLibrary,
-                      completionHandler: @escaping (Bool) -> Void) {
-        guard let allFiles = MLFile.allFiles() as? [MLFile] else {
-            assertionFailure("MediaLibraryService: Migration: Unable to retrieve all files")
-            completionHandler(false)
-            return
-        }
-
-        for media in allFiles {
-            if let newMedia = fetchMedia(with: media.url) {
-                newMedia.updateTitle(media.title)
-                newMedia.setPlayCount(media.playCount.uint32Value)
-                newMedia.setMetadataOf(.progress, intValue: media.lastPosition.int64Value)
-                newMedia.setMetadataOf(.seen, intValue: media.unread.int64Value)
-                // Only delete files that are not in playlist
-                if media.labels.isEmpty {
-                    oldMedialibrary.remove(media)
-                }
-            }
-        }
-
-        oldMedialibrary.save {
-            success in
-            completionHandler(success)
-        }
-    }
-
-    // This private method migrates old playlist and removes file and playlist
-    // from the old medialibrary.
-    // Note: This removes **only** files that are in a playlist
-    func migratePlaylists(_ oldMedialibrary: MLMediaLibrary,
-                          completionHandler: @escaping (Bool) -> Void) {
-        guard let allLabels = MLLabel.allLabels() as? [MLLabel] else {
-            assertionFailure("MediaLibraryService: Migration: Unable to retrieve all labels")
-            completionHandler(false)
-            return
-        }
-
-        for label in allLabels {
-            guard let newPlaylist = createPlaylist(with: label.name) else {
-                assertionFailure("MediaLibraryService: Migration: Unable to create playlist.")
-                continue
-            }
-
-            guard let files = label.files as? Set<MLFile> else {
-                assertionFailure("MediaLibraryService: Migration: Unable to retrieve files from label")
-                oldMedialibrary.remove(label)
-                continue
-            }
-
-            for file in files {
-                if let newMedia = fetchMedia(with: file.url) {
-                    if newPlaylist.appendMedia(withIdentifier: newMedia.identifier()) {
-                        oldMedialibrary.remove(file)
-                    }
-                }
-            }
-            oldMedialibrary.remove(label)
-        }
-        oldMedialibrary.save() {
-            success in
-            completionHandler(success)
-        }
-    }
-
-    func migrateToNewMediaLibrary(completionHandler: @escaping (Bool) -> Void) {
-        guard let oldMedialibrary = MLMediaLibrary.sharedMediaLibrary() as? MLMediaLibrary else {
-            assertionFailure("MediaLibraryService: Migration: Unable to retrieve old medialibrary")
-            completionHandler(false)
-            return
-        }
-
-        migrateMedia(oldMedialibrary) {
-            [unowned self] success in
-
-            guard success else {
-                assertionFailure("MediaLibraryService: Failed to migrate Media.")
-                completionHandler(false)
-                return
-            }
-
-            self.migratePlaylists(oldMedialibrary) {
-                [unowned self] success in
-                if success {
-                    UserDefaults.standard.set(true, forKey: MediaLibraryService.migrationKey)
-                    self.didMigrate = true
-                } else {
-                    assertionFailure("MediaLibraryService: Failed to migrate Playlist.")
-                }
-                completionHandler(success)
-            }
-        }
-    }
-}
-
 // MARK: - Helpers
 
 @objc extension MediaLibraryService {
@@ -408,8 +292,14 @@ private extension MediaLibraryService {
         mlMedia.titleIndex = Int64(player.indexOfCurrentTitle)
 
         if mlMedia.type() == .video {
-            mlMedia.requestThumbnail(of: .thumbnail, desiredWidth: 320,
-                                     desiredHeight: 200, atPosition: player.playbackPosition)
+            if let thumbnailURL = mlMedia.thumbnail() {
+                if mlMedia.progress < 0.90 {
+                    _ = try? FileManager.default.removeItem(at: thumbnailURL)
+                    VLCThumbnailsCache.invalidateThumbnail(for: thumbnailURL)
+                }
+            }
+            mlMedia.requestThumbnail(of: .thumbnail, desiredWidth: desiredThumbnailWidth,
+                                     desiredHeight: desiredThumbnailHeight, atPosition: player.playbackPosition)
         }
     }
 
@@ -446,6 +336,26 @@ private extension MediaLibraryService {
             }
         }
     }
+
+    @objc func exportMediaLibrary() {
+        guard let documentPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first,
+            let libraryPath = NSSearchPathForDirectoriesInDomains(.libraryDirectory, .userDomainMask, true).first else {
+                preconditionFailure("MediaLibraryService: Unable to find medialibrary.")
+        }
+
+        let databasePath = libraryPath + "/MediaLibrary/" + MediaLibraryService.databaseName
+        let targetPath = documentPath + "/Logs/" + MediaLibraryService.databaseName
+
+        do {
+            try FileManager.default.createDirectory(atPath: targetPath,
+                                                    withIntermediateDirectories: true)
+        } catch let error as NSError {
+            assertionFailure("Failed to create directory: \(error.localizedDescription)")
+        }
+
+        _ = try? FileManager.default.removeItem(atPath: targetPath)
+        _ = try? FileManager.default.copyItem(atPath: databasePath, toPath: targetPath)
+    }
 }
 
 // MARK: - Application notifications
@@ -471,7 +381,7 @@ private extension MediaLibraryService {
 
 @objc extension MediaLibraryService {
     func artists(sortingCriteria sort: VLCMLSortingCriteria = .alpha,
-                 desc: Bool = false, listAll all: Bool = false) -> [VLCMLArtist] {
+                 desc: Bool = false, listAll all: Bool = true) -> [VLCMLArtist] {
         return medialib.artists(with: sort, desc: desc, all: all) ?? []
     }
 
@@ -494,7 +404,7 @@ extension MediaLibraryService {
             assertionFailure("MediaLibraryService: requestThumbnail: unknown case.")
         }
 
-        if !media.requestThumbnail(of: .thumbnail, desiredWidth: 320, desiredHeight: 200, atPosition: 0.03) {
+        if !media.requestThumbnail(of: .thumbnail, desiredWidth: desiredThumbnailWidth, desiredHeight: desiredThumbnailHeight, atPosition: 0.03) {
             assertionFailure("MediaLibraryService: Failed to generate thumbnail for: \(media.identifier())")
         }
     }
@@ -742,9 +652,6 @@ extension MediaLibraryService {
     }
 
     func medialibrary(_ medialibrary: VLCMediaLibrary, didUpdateParsingStatsWithPercent percent: UInt32) {
-        if didFinishDiscovery && percent == 100 {
-             startMigrationIfNeeded()
-        }
     }
 }
 
