@@ -12,6 +12,10 @@
 import UIKit
 import LocalAuthentication
 
+extension Notification.Name {
+    static let VLCDisableGroupingDidChangeNotification = Notification.Name("disableGroupingDidChangeNotfication")
+}
+
 class SettingsController: UITableViewController {
 
     private let cellReuseIdentifier = "settingsCell"
@@ -21,17 +25,18 @@ class SettingsController: UITableViewController {
     private let notificationCenter = NotificationCenter.default
     private let actionSheet = ActionSheet()
     private let specifierManager = ActionSheetSpecifier()
-    private var mediaLibraryService = MediaLibraryService()
+    private var mediaLibraryService: MediaLibraryService
     private var localeDictionary = NSDictionary()
     private var isBackingUp = false
+    private let isLabActivated: Bool = true
 
     override var preferredStatusBarStyle: UIStatusBarStyle {
         return PresentationTheme.current.colors.statusBarStyle
     }
 
     init(mediaLibraryService: MediaLibraryService) {
-        super.init(style: .grouped)
         self.mediaLibraryService = mediaLibraryService
+        super.init(style: .grouped)
         self.mediaLibraryService.deviceBackupDelegate = self
         setup()
     }
@@ -133,7 +138,7 @@ class SettingsController: UITableViewController {
         if #available(iOS 10, *) {
             ImpactFeedbackGenerator().selectionChanged()
         }
-        let vc = VLCStoreViewController()
+        let vc = StoreViewController(nibName: "VLCStoreViewController", bundle: nil)
         let storeVC = UINavigationController(rootViewController: vc)
         present(storeVC, animated: true, completion: nil)
     }
@@ -142,6 +147,7 @@ class SettingsController: UITableViewController {
         self.view.backgroundColor = PresentationTheme.current.colors.background
         setNavBarAppearance()
         self.setNeedsStatusBarAppearanceUpdate()
+        self.tableView.reloadData() // When theme changes hide the black theme section if needed
     }
 
     @objc private func miniPlayerIsShown() {
@@ -222,12 +228,21 @@ class SettingsController: UITableViewController {
             ImpactFeedbackGenerator().selectionChanged()
         }
     }
+
+    private func exportMediaLibrary() {
+        self.mediaLibraryService.exportMediaLibrary()
+    }
 }
 
 extension SettingsController {
 
     override func numberOfSections(in tableView: UITableView) -> Int {
-        return SettingsSection.allCases.count
+        var numberOfSections = SettingsSection.allCases.count
+        // Remove the last section if the lab is deactivated
+        if isLabActivated == false {
+            numberOfSections = numberOfSections - 1
+        }
+        return numberOfSections
     }
 
     override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
@@ -251,7 +266,9 @@ extension SettingsController {
         case 8:
             return MediaLibraryOptions.allCases.count
         case 9:
-            return FileSyncOptions.allCases.count
+            return NetworkOptions.allCases.count
+        case 10:
+            return Lab.allCases.count
         default:
             return 0
         }
@@ -274,7 +291,15 @@ extension SettingsController {
         switch section {
         case .main:
             let main = MainOptions(rawValue: indexPath.row)
+            let brightThemeForced = userDefaults.integer(forKey: kVLCSettingAppTheme) == kVLCSettingAppThemeBright
+            if indexPath.row == MainOptions.blackTheme.rawValue {
+                if brightThemeForced {
+                    //If the user wants the bright theme, we hide black theme settings
+                    cell.isHidden = true
+                }
+            }
             cell.sectionType = main
+            cell.blackThemeSwitchDelegate = self
         case .generic:
             let generic = GenericOptions(rawValue: indexPath.row)
             cell.sectionType = generic
@@ -312,6 +337,7 @@ extension SettingsController {
                 cell.mainLabel.textColor = PresentationTheme.current.colors.orangeUI
             }
             cell.mediaLibraryBackupSwitchDelegate = self
+            cell.medialibraryDisableGroupingSwitchDelegate = self
             if indexPath.row == MediaLibraryOptions.includeMediaLibInDeviceBackup.rawValue {
                 if isBackingUp {
                     cell.accessoryView = .none
@@ -327,9 +353,16 @@ extension SettingsController {
                 cell.accessoryView = .none
                 cell.accessoryType = .none
             }
-        case .fileSync:
-            let fileSyncOptions = FileSyncOptions(rawValue: indexPath.row)
-            cell.sectionType = fileSyncOptions
+        case .network:
+            let networkOptions = NetworkOptions(rawValue: indexPath.row)
+            cell.sectionType = networkOptions
+        case .lab:
+            let lab = Lab(rawValue: indexPath.row)
+            cell.sectionType = lab
+            if indexPath.row == 1 {
+                cell.accessoryView = .none
+                cell.accessoryType = .none
+            }
         }
         return cell
     }
@@ -345,6 +378,10 @@ extension SettingsController {
             forceRescanAlert()
             return
         }
+        if section == .lab && indexPath.row == 1 {
+            exportMediaLibrary()
+            return
+        }
         switch section {
         case .main:
             let mainSection = MainOptions(rawValue: indexPath.row)
@@ -370,13 +407,17 @@ extension SettingsController {
         case .audio:
             break
         case .casting:
-            break
+            let castingSection = CastingOptions(rawValue: indexPath.row)
+            playHaptics(sectionType: castingSection)
+            showActionSheet(for: castingSection)
         case .mediaLibrary:
             break
-        case .fileSync:
-            let fileSection = FileSyncOptions(rawValue: indexPath.row)
-            playHaptics(sectionType: fileSection)
-            showActionSheet(for: fileSection)
+        case .network:
+            let networkSection = NetworkOptions(rawValue: indexPath.row)
+            playHaptics(sectionType: networkSection)
+            showActionSheet(for: networkSection)
+        case .lab:
+            break
         }
     }
 
@@ -406,6 +447,10 @@ extension SettingsController {
 
     override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
         let automaticDimension = UITableView.automaticDimension
+        if indexPath == [SettingsSection.main.rawValue, MainOptions.blackTheme.rawValue] {
+            let brightThemeForced = userDefaults.integer(forKey: kVLCSettingAppTheme) == kVLCSettingAppThemeBright
+            return brightThemeForced ? 0 : automaticDimension //If the user wants the bright theme, we hide black theme settings
+        }
         if indexPath == [SettingsSection.privacy.rawValue, PrivacyOptions.enableBiometrics.rawValue] {
             let isPasscodeOn = userDefaults.bool(forKey: kVLCSettingPasscodeOnKey)
             let privacySection = PrivacyOptions(rawValue: indexPath.row)
@@ -452,6 +497,14 @@ extension SettingsController: MediaLibraryHidingDelegate {
     }
 }
 
+// MARK: - SwitchOn Delegates
+
+extension SettingsController: BlackThemeActivateDelegate {
+    func blackThemeSwitchOn(state: Bool) {
+        PresentationTheme.themeDidUpdate()
+    }
+}
+
 extension SettingsController: PasscodeActivateDelegate {
 
     func passcodeLockSwitchOn(state: Bool) {
@@ -479,3 +532,9 @@ extension SettingsController: MediaLibraryBackupActivateDelegate {
         mediaLibraryService.excludeFromDeviceBackup(state)
     }
 }
+
+extension SettingsController: MediaLibraryDisableGroupingDelegate {
+    func medialibraryDisableGroupingSwitchOn(state: Bool) {
+        notificationCenter.post(name: .VLCDisableGroupingDidChangeNotification, object: self)
+    }
+}
